/* OSBeeWiFi Firmware
 *
 * OSBeeWiFi library
 * December 2016 @ bee.opensprinkler.com
 *
 * This file is part of the OSBeeWiFi library
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "OSBeeWiFi.h"
#include "font.h"
#include "images.h"
#include "program.h"

// Dibawah ini Ana tambahkan untuk meng aktifkan SPI Mode di 74HC595
#include <SPI.h>  


byte OSBeeWiFi::state = OSB_STATE_INITIAL;
byte OSBeeWiFi::has_rtc = false;
byte OSBeeWiFi::curr_zbits = 0;
byte OSBeeWiFi::next_zbits = 0;
byte OSBeeWiFi::program_busy = 0;
File OSBeeWiFi::log_file;
File OSBeeWiFi::prog_file;
ulong OSBeeWiFi::open_tstamp[]={0,0,0};
ulong OSBeeWiFi::curr_utc_time = 0;
SSD1306 OSBeeWiFi::display(0x3c, SDA, SCL);

const char* config_fname = CONFIG_FNAME;
const char* log_fname = LOG_FNAME;
const char* prog_fname = PROG_FNAME;

extern ProgramData pd;

/* Options name, default integer value, max value, default string value
 * Integer options don't have string value
 * String options don't have integer or max value
 * ini ana tambahkan zone 4 sampai 8
 */
OptionStruct OSBeeWiFi::options[] = {
  {"fwv", OSB_FWV,     255, ""},
  {"tmz", 76,          104, ""},   // default 48, 104, "" , 48 = GMT 0 , 16 = GMT -8 dan 76 = GMT +7
  {"ntp", OSB_NTP_D, 255, ""},     // Ini Ana tambahkan untuk NTP
  {"htp", 80,        65535, ""},
  {"mod", OSB_MOD_AP,  255, ""},
  {"ssid", 0, 0, ""},  // string options have 0 max value
  {"pass", 0, 0, ""},
  {"auth", 0, 0, ""},
  {"dkey", 0, 0, DEFAULT_DKEY},
  {"name", 0, 0, DEFAULT_NAME},
  {"zon0", 0, 0, "Zone 1"},
  {"zon1", 0, 0, "Zone 2"},
  {"zon2", 0, 0, "Zone 3"},
  {"zon3", 0, 0, "Zone 4"},
  {"zon4", 0, 0, "Zone 5"},
  {"zon5", 0, 0, "Zone 6"},
  {"zon6", 0, 0, "Zone 7"},
  {"zon7", 0, 0, "Zone 8"},
};

ulong OSBeeWiFi::curr_loc_time() {
  return curr_utc_time + (ulong)options[OPTION_TMZ].ival*900L - 43200L;
}

/** Calculate local time (UTC time plus time zone offset) */ 
/** Ini ana tambahkan fungsi os.now_tz */
time_t OSBeeWiFi::now_tz() {
  return now()+(int32_t)3600/4*(ulong)(options[OPTION_TMZ].ival-48);
}


void OSBeeWiFi::begin() {

  SPI.setDataMode(SPI_MODE0);
  SPI.setBitOrder(MSBFIRST);
  SPI.setClockDivider(SPI_CLOCK_DIV16);
  SPI.setHwCs(true);
  SPI.begin();
  SPI.transfer(0); // matikan semua zone

  state = OSB_STATE_INITIAL;
  
  if(!SPIFFS.begin()) {
    DEBUG_PRINTLN(F("\nSPIFFS Mount Failed! "));
  } else {
    DEBUG_PRINTLN(F("\nSPIFFS Mount succesfull. "));
  }
  
  display.init();
  display.flipScreenVertically();
  
  flashScreen();

  if(RTC.detect()) {
    has_rtc = true;
    DEBUG_PRINTLN("\nRTC detected");
  }
  else {
    has_rtc = false;
    DEBUG_PRINTLN("\nRTC is NOT detected!");
  }
}

void OSBeeWiFi::boldFont(bool bold) {
  if(bold) display.setFont(Noto_Sans_Bold_12);
  else display.setFont(Noto_Sans_12);

}

void OSBeeWiFi::flashScreen() {
  boldFont(true);
  display.drawString(0, 0, "OpenSprinkler Bee Mod");
  display.drawXbm(34, 24, WiFi_Logo_width, WiFi_Logo_height, WiFi_Logo_bits);
  display.display();
  boldFont(false);  
}

void OSBeeWiFi::options_setup() {
  if(!SPIFFS.exists(config_fname)) { // if config file does not exist
    options_save(); // save default option values
    return;
  } 
  options_load();
  
  if(options[OPTION_FWV].ival != OSB_FWV)  {
    // if firmware version has changed
    // re-save options, thus preserving
    // shared options with previous firmwares
    options[OPTION_FWV].ival = OSB_FWV;
    options_save();
    return;
  }
}

void OSBeeWiFi::options_reset() {
  DEBUG_PRINT(F("reset to factory default..."));
  if(!SPIFFS.remove(config_fname)) {
    DEBUG_PRINT(F("failed to remove config file"));
    return;
  }
  DEBUG_PRINTLN(F("ok"));
}

void OSBeeWiFi::log_reset() {
  if(!SPIFFS.remove(log_fname)) {
    DEBUG_PRINTLN(F("failed to remove log file"));
    return;
  }
}

void OSBeeWiFi::progs_reset() {
  if(!SPIFFS.remove(prog_fname)) {
    DEBUG_PRINTLN(F("failed to remove program file"));
    return;
  }
  DEBUG_PRINTLN(F("ok"));    
}

int8_t OSBeeWiFi::find_option(String name) {
  for(byte i=0;i<NUM_OPTIONS;i++) {
    if(name == options[i].name) {
      return i;
    }
  }
  return -1;
}

void OSBeeWiFi::options_load() {
  File file = SPIFFS.open(config_fname, "r");
  DEBUG_PRINT(F("\nLoading config file from SPIFFS..."));
  if(!file) {
    DEBUG_PRINTLN(F("\nLoading config file from SPIFFS failed!"));
    return;
  }
  while(file.available()) {
    String name = file.readStringUntil(':');
    String sval = file.readStringUntil('\n');
    sval.trim();
    int8_t idx = find_option(name);
    if(idx<0) continue;
    if(options[idx].max) {  // this is an integer option
      options[idx].ival = sval.toInt();
    } else {  // this is a string option
      options[idx].sval = sval;
    }
  }
  DEBUG_PRINTLN(F("ok"));
  file.close();
}

void OSBeeWiFi::options_save() {
  File file = SPIFFS.open(config_fname, "w");
  DEBUG_PRINT(F("\nSaving config file..."));  

  if(!file) {
    DEBUG_PRINTLN(F("failed"));
    return;
  }
  OptionStruct *o = options;
  for(byte i=0;i<NUM_OPTIONS;i++,o++) {
    file.print(o->name + ":");
    if(o->max)
      file.println(o->ival);
    else
      file.println(o->sval);
  }
  DEBUG_PRINTLN(F("ok"));  
  file.close();

  DEBUG_PRINT(F("Time Zone: "));
  DEBUG_PRINTLN(options[OPTION_TMZ].ival);
  DEBUG_PRINT(F("Status NTP: "));
  DEBUG_PRINTLN(options[OPTION_NTP].ival);
}

void OSBeeWiFi::toggle_led() {
  static byte status = 0;
  status = 1-status;
  set_led(!status);
}

void OSBeeWiFi::set_led(byte status) {
  display.setColor(status ? WHITE : BLACK);
  display.fillCircle(122, 58, 4);
  display.display();
  display.setColor(WHITE);
}

bool OSBeeWiFi::get_cloud_access_en() {
  if(options[OPTION_AUTH].sval.length()==32) {
    return true;
  }
  return false;
}

void OSBeeWiFi::write_log(const LogStruct& data) {
  File file;
  uint curr = 0;
  DEBUG_PRINT(F("saving log data..."));  
  if(!SPIFFS.exists(log_fname)) {  // create log file
    file = SPIFFS.open(log_fname, "w");
    if(!file) {
      DEBUG_PRINTLN(F("failed"));
      return;
    }
    // fill log file
    uint next = curr+1;
    file.write((const byte*)&next, sizeof(next));
    file.write((const byte*)&data, sizeof(LogStruct));
    LogStruct l;
    l.tstamp = 0;
    for(;next<MAX_LOG_RECORDS;next++) {
      file.write((const byte*)&l, sizeof(LogStruct));
    }
  } else {
    file = SPIFFS.open(log_fname, "r+");
    if(!file) {
      DEBUG_PRINTLN(F("failed"));
      return;
    }
    file.readBytes((char*)&curr, sizeof(curr));
    uint next = (curr+1) % MAX_LOG_RECORDS;
    file.seek(0, SeekSet);
    file.write((const byte*)&next, sizeof(next));

    file.seek(sizeof(curr)+curr*sizeof(LogStruct), SeekSet);
    file.write((const byte*)&data, sizeof(LogStruct));
  }
  DEBUG_PRINTLN(F("ok"));      
  file.close();
}

bool OSBeeWiFi::read_log_start() {
  if(log_file) log_file.close();
  log_file = SPIFFS.open(log_fname, "r");
  if(!log_file) return false;
  uint curr;
  if(log_file.readBytes((char*)&curr, sizeof(curr)) != sizeof(curr)) return false;
  if(curr>=MAX_LOG_RECORDS) return false;
  return true;
}

bool OSBeeWiFi::read_log_next(LogStruct& data) {
  if(!log_file) return false;
  if(log_file.readBytes((char*)&data, sizeof(LogStruct)) != sizeof(LogStruct)) return false;
  return true;  
}

bool OSBeeWiFi::read_log_end() {
  if(!log_file) return false;
  log_file.close();
  return true;
}

void OSBeeWiFi::apply_zbits() {
  static LogStruct l;
  for(byte i=0;i<MAX_NUMBER_ZONES;i++) {
    byte mask = (byte)1<<i;
    if(next_zbits & mask) {
      if(curr_zbits & mask)     continue; // bit is already set
      open(i); // open solenoid
      open_tstamp[i]=curr_utc_time;
      l.tstamp = curr_utc_time;
      l.dur = 0;
      l.event = 'o';
      l.zid = i;
      l.pid = pd.curr_prog_index;
      l.tid = pd.curr_task_index;
      write_log(l);

      DEBUG_PRINT("Nilai Decimal Open: ");
      Serial.println(next_zbits);
      SPI.transfer(next_zbits);

    } else {
      if(!(curr_zbits & mask))  continue; // bit is already reset
      close(i);  // close solenoid
      l.tstamp = curr_utc_time;
      l.dur = curr_utc_time-open_tstamp[i]+1;
      l.event = 'c';
      l.zid = i;
      l.pid = pd.curr_prog_index;
      l.tid = pd.curr_task_index;
      write_log(l);
      open_tstamp[i]=0;

      DEBUG_PRINT("Nilai Decimal Close: ");
      Serial.println(next_zbits);
      SPI.transfer(next_zbits);

    }
  }
  curr_zbits = next_zbits;  // update curr_zbits
}

void OSBeeWiFi::set_zone(byte zid, byte value) {
  if(zid>=MAX_NUMBER_ZONES)  return; // out of bound
  byte mask = (byte)1<<zid;
  if(value) next_zbits = next_zbits | mask;
  else      next_zbits = next_zbits & (~mask);

}


// open a zone by zone index
void OSBeeWiFi::open(byte zid) {
  DEBUG_PRINT("open_nl ");
  DEBUG_PRINTLN(zid);
}

// close a zone
void OSBeeWiFi::close(byte zid) {
  DEBUG_PRINT("close_nl ");
  DEBUG_PRINTLN(zid);   
}


